unit CliProcess;

interface
uses Srv_Cli, WSocket, WSocketS, sysutils, Classes, db;

type
  TClientConnBuffer = class;
  TSrvClient = class(TWSocketClient)
  public
    Buffer : TConnBuffer;
    constructor Create(AOwner:TComponent); override;
    destructor  Destroy; override;
  end;
  TClientConnBuffer = class(TConnBuffer)
  protected
    FDataSet: TDataSet;
    procedure ProcessData; override;
    procedure do_cmdLoadFile;
    procedure do_cmdWho;
    procedure do_cmdError(const Msg: String);
    procedure do_cmdLogin;
  public
  
    SessionName, DatabaseName : String;
    constructor Create( Socket: TWSocket ); override;
    destructor  Destroy; override;
    procedure   Log(const Msg:String); override;

    procedure   AddSQLParam( const ParamName: String; ParamType: TFieldType; const Value: String);
    function    OpenSQL( DataSet: TDataset; const SQL: String; HasParams: Boolean=False ):Integer;
    function    UpdateFieldDefs( DataSet: TDataset; const SQL: String; HasParams: Boolean=False ):Integer;
    function    ExecSQL( DataSet: TDataset; const SQL: String; HasParams: Boolean=False ):Integer;
    function    RunSQL:Integer;

  end;

var
  ClientLog:TStrings=nil;

implementation

constructor TSrvClient.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  Buffer:=TClientConnBuffer.Create( self );
end;

destructor TSrvClient.Destroy;
begin
  Buffer.Free;
  inherited Destroy;
end;

constructor TClientConnBuffer.Create( Socket: TWSocket );
begin
  inherited Create( Socket );
  FDataSet := nil;
end;

destructor TClientConnBuffer.Destroy;
begin
  inherited Destroy;
end;

procedure TClientConnBuffer.Log(const Msg:String);
begin
  if ClientLog<>nil then
    ClientLog.add( msg)
end;

procedure TClientConnBuffer.ProcessData;
begin

  Log ( 'Command: '+CommandNames[Command].Name );
  case Command of
    cmdError: begin
      do_cmdError( ReadParam );
    end;
    cmdLogout: begin
//      FSocket.CloseDelayed;
    end;
    cmdLogin: begin
      do_cmdLogin;
    end;
    cmdTime: begin
      WriteParam( DateTimeToStr(Now) );
      Send;
    end;
    cmdLoadFile: begin
      do_cmdLoadFile;
    end;
    cmdWho: begin
      do_cmdWho;
    end
  end;
end;

function TClientConnBuffer.OpenSQL;
begin

  if FDataSet<>nil then
    Error('ClientConnectBuffer j esta em uso');
  FDataSet := DataSet;

  SetCommand( cmdSQL );
  WriteByte( Byte(cmdSQLOpen) );
  WriteParam( SQL );
  if not HasParams then Result := RunSQL;

end;

function TClientConnBuffer.UpdateFieldDefs;
begin

  if FDataSet<>nil then
    Error('ClientConnectBuffer j esta em uso');
  FDataSet := DataSet;

  SetCommand( cmdSQL );
  WriteByte( Byte(cmdSQLFieldDefs) );
  WriteParam( SQL );
  if not HasParams then Result := RunSQL;

end;

function TClientConnBuffer.ExecSQL;
var
  ExecOnly:Boolean;
  S1:String;
  fazer_params: integer;
begin

  if FDataSet<>nil then
    Error('ClientConnectBuffer j esta em uso');
  FDataSet := DataSet;

  SetCommand( cmdSQL );
  WriteByte( Byte(cmdSQLExec) );
  WriteParam( SQL );

  if not HasParams then Result := RunSQL;

end;

procedure TClientConnBuffer.AddSQLParam( const ParamName: String; ParamType: TFieldType; const Value: String);
begin
  WriteByte( 0 ); // HasParam;
  WriteParam( ParamName + ';' + inttostr(Integer( ParamType) ) );
  WriteParam( Value );
end;


function TClientConnBuffer.RunSQL;
var
  I, _FieldCount: Integer;
  fazer_params: integer;
  S, Name: String;
  Datatype: TFieldType;
  Size: Integer;
  Required: Boolean;
  SQLCommand: TSQLCommand;
begin

  WriteByte( 1 ); // Acabou Query.Params

  Send; Wait( cmdSQL, 30000 );

  try
    SQLCommand := TSQLCommand (ReadByte);
    if SQLCommand=cmdSQLExec then begin
      Result   := ReadInt;
      FDataSet := nil;
    end
    else begin
      FDataSet.DisableControls;
      try

        FDataSet.Close;
        FDataSet.FieldDefs.Clear;

        _FieldCount := ReadInt;

        for I:=0 to _FieldCount-1 do begin
          S := ReadParam;
          Name     := strDivide( S, ';');
          Datatype := TFieldType(StrToInt( strDivide( S, ';') ));
          Size     := StrToInt( strDivide( S, ';') );
          Required := Boolean( StrToInt( strDivide( S, ';') ));
          FDataSet.FieldDefs.Add(Name, DataType, Size, Required);
        end;

        if SQLCommand=cmdSQLOpen then begin

          FDataSet.Open;

          while ReadByte=0 do begin
            FDataSet.Append;
            for I:=0 to _FieldCount-1 do begin
              FDataSet.Fields[I].asString := ReadParam;
            end;
            FDataSet.Post;
            Inc ( Result );
          end;

          FDataSet.First;

        end;

        if Result <> ReadInt then
          Error('Erro na Trailer. Recnum');

      finally
        FDataSet.EnableControls;
        FDataSet := nil;
      end;
    end;
  except
    on E:Exception do begin
      do_cmdError(E.ClassName+': '+E.Message);
    end;
  end
end;

procedure TClientConnBuffer.do_cmdLoadFile;
begin
end;

procedure TClientConnBuffer.do_cmdWho;
var
  I       : Integer;
  AClient : TSrvClient;
begin
end;

procedure TClientConnBuffer.do_cmdError(const Msg:String);
begin
  if Command=cmdError then
    raise exception.create( ReadParam + #13 + Msg )
  else
    raise exception.create( Msg );
end;

procedure TClientConnBuffer.do_cmdLogin();
begin
end;


end.
 